@@ -2,25 +2,28 @@
 #
 # Author: Dan Garrette <dhgarrette@gmail.com>
 #
-# Copyright (C) 2001-2012 NLTK Project
-# URL: <http://www.nltk.org>
+# Copyright (C) 2001-2016 NLTK Project
+# URL: <http://nltk.org>
 # For license information, see LICENSE.TXT
 
 """
 Module for a resolution-based First Order theorem prover.
 """
+from __future__ import print_function, unicode_literals
 
 import operator
 from collections import defaultdict
+from functools import reduce
 
 from nltk.sem import skolemize
 from nltk.sem.logic import (VariableExpression, EqualityExpression,
-                            ApplicationExpression, LogicParser,
+                            ApplicationExpression, Expression,
                             NegatedExpression, Variable,
                             AndExpression, unique_variable, OrExpression,
                             is_indvar, IndividualVariableExpression, Expression)
 
 from nltk.inference.api import Prover, BaseProverCommand
+from nltk.compat import python_2_unicode_compatible
 
 class ProverParseError(Exception): pass
 
@@ -47,14 +50,14 @@ class ResolutionProver(Prover):
                 clauses.extend(clausify(a))
             result, clauses = self._attempt_proof(clauses)
             if verbose:
-                print ResolutionProverCommand._decorate_clauses(clauses)
-        except RuntimeError, e:
+                print(ResolutionProverCommand._decorate_clauses(clauses))
+        except RuntimeError as e:
             if self._assume_false and str(e).startswith('maximum recursion depth exceeded'):
                 result = False
                 clauses = []
             else:
                 if verbose:
-                    print e
+                    print(e)
                 else:
                     raise e
         return (result, clauses)
@@ -154,6 +157,7 @@ class ResolutionProverCommand(BaseProverCommand):
             out += '[%s] %s %s %s\n' % (seq, clauses[i], parents, taut)
         return out
 
+@python_2_unicode_compatible
 class Clause(list):
     def __init__(self, data):
         list.__init__(self, data)
@@ -297,10 +301,10 @@ class Clause(list):
         return Clause([atom.substitute_bindings(bindings) for atom in self])
 
     def __str__(self):
-        return '{' + ', '.join([str(item) for item in self]) + '}'
+        return '{' + ', '.join("%s" % item for item in self) + '}'
 
     def __repr__(self):
-        return str(self)
+        return "%s" % self
 
 def _iterate_first(first, second, bindings, used, skipped, finalize_method, debug):
     """
@@ -457,6 +461,7 @@ def _clausify(expression):
     raise ProverParseError()
 
 
+@python_2_unicode_compatible
 class BindingDict(object):
     def __init__(self, binding_list=None):
         """
@@ -543,10 +548,11 @@ class BindingDict(object):
         return len(self.d)
 
     def __str__(self):
-        return '{' + ', '.join(['%s: %s' % (v, self.d[v]) for v in self.d]) + '}'
+        data_str = ', '.join('%s: %s' % (v, self.d[v]) for v in sorted(self.d.keys()))
+        return '{' + data_str + '}'
 
     def __repr__(self):
-        return str(self)
+        return "%s" % self
 
 
 def most_general_unification(a, b, bindings=None):
@@ -604,7 +610,7 @@ class DebugObject(object):
 
     def line(self, line):
         if self.enabled:
-            print '    '*self.indent + line
+            print('    '*self.indent + line)
 
 
 def testResolutionProver():
@@ -623,59 +629,59 @@ def testResolutionProver():
     resolution_test('-all x.some y.F(x,y) & some x.all y.(-F(x,y))')
     resolution_test('some x.all y.sees(x,y)')
 
-    p1 = LogicParser().parse(r'all x.(man(x) -> mortal(x))')
-    p2 = LogicParser().parse(r'man(Socrates)')
-    c = LogicParser().parse(r'mortal(Socrates)')
-    print '%s, %s |- %s: %s' % (p1, p2, c, ResolutionProver().prove(c, [p1,p2]))
+    p1 = Expression.fromstring(r'all x.(man(x) -> mortal(x))')
+    p2 = Expression.fromstring(r'man(Socrates)')
+    c = Expression.fromstring(r'mortal(Socrates)')
+    print('%s, %s |- %s: %s' % (p1, p2, c, ResolutionProver().prove(c, [p1,p2])))
 
-    p1 = LogicParser().parse(r'all x.(man(x) -> walks(x))')
-    p2 = LogicParser().parse(r'man(John)')
-    c = LogicParser().parse(r'some y.walks(y)')
-    print '%s, %s |- %s: %s' % (p1, p2, c, ResolutionProver().prove(c, [p1,p2]))
+    p1 = Expression.fromstring(r'all x.(man(x) -> walks(x))')
+    p2 = Expression.fromstring(r'man(John)')
+    c = Expression.fromstring(r'some y.walks(y)')
+    print('%s, %s |- %s: %s' % (p1, p2, c, ResolutionProver().prove(c, [p1,p2])))
 
-    p = LogicParser().parse(r'some e1.some e2.(believe(e1,john,e2) & walk(e2,mary))')
-    c = LogicParser().parse(r'some e0.walk(e0,mary)')
-    print '%s |- %s: %s' % (p, c, ResolutionProver().prove(c, [p]))
+    p = Expression.fromstring(r'some e1.some e2.(believe(e1,john,e2) & walk(e2,mary))')
+    c = Expression.fromstring(r'some e0.walk(e0,mary)')
+    print('%s |- %s: %s' % (p, c, ResolutionProver().prove(c, [p])))
 
 def resolution_test(e):
-    f = LogicParser().parse(e)
+    f = Expression.fromstring(e)
     t = ResolutionProver().prove(f)
-    print '|- %s: %s' % (f, t)
+    print('|- %s: %s' % (f, t))
 
 def test_clausify():
-    lp = LogicParser()
+    lexpr = Expression.fromstring
 
-    print clausify(lp.parse('P(x) | Q(x)'))
-    print clausify(lp.parse('(P(x) & Q(x)) | R(x)'))
-    print clausify(lp.parse('P(x) | (Q(x) & R(x))'))
-    print clausify(lp.parse('(P(x) & Q(x)) | (R(x) & S(x))'))
+    print(clausify(lexpr('P(x) | Q(x)')))
+    print(clausify(lexpr('(P(x) & Q(x)) | R(x)')))
+    print(clausify(lexpr('P(x) | (Q(x) & R(x))')))
+    print(clausify(lexpr('(P(x) & Q(x)) | (R(x) & S(x))')))
 
-    print clausify(lp.parse('P(x) | Q(x) | R(x)'))
-    print clausify(lp.parse('P(x) | (Q(x) & R(x)) | S(x)'))
+    print(clausify(lexpr('P(x) | Q(x) | R(x)')))
+    print(clausify(lexpr('P(x) | (Q(x) & R(x)) | S(x)')))
 
-    print clausify(lp.parse('exists x.P(x) | Q(x)'))
+    print(clausify(lexpr('exists x.P(x) | Q(x)')))
 
-    print clausify(lp.parse('-(-P(x) & Q(x))'))
-    print clausify(lp.parse('P(x) <-> Q(x)'))
-    print clausify(lp.parse('-(P(x) <-> Q(x))'))
-    print clausify(lp.parse('-(all x.P(x))'))
-    print clausify(lp.parse('-(some x.P(x))'))
+    print(clausify(lexpr('-(-P(x) & Q(x))')))
+    print(clausify(lexpr('P(x) <-> Q(x)')))
+    print(clausify(lexpr('-(P(x) <-> Q(x))')))
+    print(clausify(lexpr('-(all x.P(x))')))
+    print(clausify(lexpr('-(some x.P(x))')))
 
-    print clausify(lp.parse('some x.P(x)'))
-    print clausify(lp.parse('some x.all y.P(x,y)'))
-    print clausify(lp.parse('all y.some x.P(x,y)'))
-    print clausify(lp.parse('all z.all y.some x.P(x,y,z)'))
-    print clausify(lp.parse('all x.(all y.P(x,y) -> -all y.(Q(x,y) -> R(x,y)))'))
+    print(clausify(lexpr('some x.P(x)')))
+    print(clausify(lexpr('some x.all y.P(x,y)')))
+    print(clausify(lexpr('all y.some x.P(x,y)')))
+    print(clausify(lexpr('all z.all y.some x.P(x,y,z)')))
+    print(clausify(lexpr('all x.(all y.P(x,y) -> -all y.(Q(x,y) -> R(x,y)))')))
 
 
 def demo():
     test_clausify()
-    print
+    print()
     testResolutionProver()
-    print
+    print()
 
-    p = LogicParser().parse('man(x)')
-    print ResolutionProverCommand(p, [p]).prove()
+    p = Expression.fromstring('man(x)')
+    print(ResolutionProverCommand(p, [p]).prove())
 
 if __name__ == '__main__':
     demo()
